# define parameters
param n  = 20;             # number of spatial discretization points
param Nt = 400;            # number of time discretization points
param N  = floor((n+1)/2); # maximal number of control impulses
param eps= 1/(n*Nt);       # relaxation parameter ${\color{comment}\varepsilon}$ in *tex\textcolor{comment}{\eqref{eq:optD6}}*/
param u0 = 1;              # parameters ${\color{comment}u^0\text{ and }u^1}$, see *tex\textcolor{comment}{\eqref{eq:y0y1}}*/
param u1 = 5;

# define variables
var y	  {k in 0..N, j in 0..Nt, i in 1..n};	# unknowns ${\color{comment}[y_k^j]_i}$
var tau {k in 0..N} >=0;   # unknown impulse times ${\color{comment}\tau_k}$ subject to *tex\textcolor{comment}{\eqref{eq:optD2}}*/

# objective function, see *tex\textcolor{comment}{\eqref{eq:optD1}}*/
minimize T:
	sum {k in 0..N} tau[k];

# define the constraints
# constraint *tex\textcolor{comment}{\eqref{eq:optD7}}*/ for interior points
subject to y_dyn {k in 0..N, j in 1..Nt, i in 2..n-1}:
	y[k,j,i]-y[k,j-1,i] =
		(n+1)^2*(y[k,j,i-1]-2*y[k,j,i]+y[k,j,i+1]+y[k,j-1,i-1]-2*y[k,j-1,i]+y[k,j-1,i+1])/2*tau[k]/Nt;
# constraint *tex\textcolor{comment}{\eqref{eq:optD7}}*/ for i=n
subject to y_dyn_Dirichlet {k in 0..N, j in 1..Nt}:
	y[k,j,n]-y[k,j-1,n] =
		(n+1)^2*(y[k,j,n-1]-2*y[k,j,n]+y[k,j-1,n-1]-2*y[k,j-1,n])/2*tau[k]/Nt;
# constraint *tex\textcolor{comment}{\eqref{eq:optD7}}*/ for i=1
subject to y_dyn_Neuman {k in 0..N, j in 1..Nt}:
	y[k,j,1]-y[k,j-1,1] =
		(n+1)^2*(y[k,j,2]-y[k,j,1]+y[k,j-1,2]-y[k,j-1,1])/2*tau[k]/Nt;
# initial condition, constraint *tex\textcolor{comment}{\eqref{eq:optD3}}*/
subject to y_init {i in 1..n}:
	y[0,0,i] = u0;
# terminal condition, constraint *tex\textcolor{comment}{\eqref{eq:optD6}}*/
subject to y_end1 {i in 1..n}:
	y[N,Nt,i] >= u1-eps;
subject to y_end2 {i in 1..n}:
	y[N,Nt,i] <= u1+eps;
# constraint *tex\textcolor{comment}{\eqref{eq:optD4}}*/
subject to continuity {k in 0..N-1, i in 1..n-1}:
	y[k+1,0,i]=y[k,Nt,i];
# constraint *tex\textcolor{comment}{\eqref{eq:optD5}}*/
subject to jump {k in 0..N-1}:
	y[k+1,0,n]>=y[k,Nt,n];

# solve the problem with IpOpt
option solver ipopt;
option ipopt_options "max_iter=100000 linear_solver=mumps halt_on_ampl_error yes";
solve;

# display parameters and solution
printf: "u0 = %24.16e\n", u0;
printf: "u1 = %24.16e\n", u1;
printf: "T  = %24.16e\n", T;
printf: "n  = %d\n", n;
printf: "Nt = %d\n", Nt;
printf: "N  = %d\n", N;
printf: "Data\n";
printf: "t_k =";
printf {k in 0..N}:  " %24.16e\t", tau[k];
printf: "\n";
printf: "y =\n";
for {k in 0..N} {
	printf "k = %d\n", k;
	printf {j in 0..Nt, i in 1..n}: " %24.16e\n", y[k,j,i];
}

end; # quit AMPL
